############################################################################
# Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2016. #
# All Rights Reserved.                                                     #
#                                                                          #
# Pre-requisite: WAS and CLM should be installed                           #
# This script updates CLM war files on a single WAS server and is meant to #
# be used when installing an iFix.                                         #
# For any .war file found in the directory passed in, it will try to find  #
# a corresponding installed application and update it.                     #
#                                                                          #
# Usage (Win):                                                             #
#  wsadmin -language jython -user <username> -password <password>          # 
#    -f <path to clm_update.py e.g.                                        #
#        C:/JazzTeamServer/server/was/clm_update.py>                       #
#    <path to war files to be updated e.g. C:/iFixPatch>                   #
#                                                                          #
# Usage (Linux):                                                           #
#  ./wsadmin.sh -language jython -user <username> -password <password>     #
#    -f <path to clm_update.py e.g.                                        #
#        /opt/IBM/JazzTeamServer/server/was/clm_update.py>                 #                                
#    <path to war files e.g. /opt/IBM/iFixPatch>                           #
############################################################################

import os
import sys
import time

# Keep track of any errors/exceptions raised during the app installation.
returncode = 0

# retrieve all .war files from directory
def constructAppsList(dir):
	global returncode
	apps = []
	if os.path.exists(dir):
		for filename in os.listdir(dir):
			if filename.endswith('.war'):
				apps.append(filename.replace(".war", ""))
			#endIf
		#endFor
	else:
		print "Error: the following directory is invalid: "+dir
		# indicate an error due to invalid directory
		returncode = 2
		return apps
		
	if not apps:
		print "Error: there are no war files in the following directory: "+dir
		# indicate an error due to missing war files
		returncode = 3
	return apps
#endDef

# Determine the offsets of the arguments
if (sys.argv[0] == '-language'):
	firstArgIndex = 2
else:
	firstArgIndex = 0

# Get the war file location from commandline
war_location = sys.argv[firstArgIndex+0]

#######################
# update all the apps #
#######################

# Get available apps from war_location
apps = constructAppsList(war_location)

installedApps = AdminApp.list()
if (installedApps != ['']):
	installedApps = installedApps [0:len(installedApps)].split(lineSeparator)
#endIf

# Keep track of apps that weren't updated
missingApps = ''
for app in apps:
	try:
		print "Updating " + app
		full_path_to_app = war_location + "/" +app + ".war"
		attrs = ['-operation', 'update', '-contents', full_path_to_app]
		
		foundApp = 0
		for installedApp in installedApps:
			if (installedApp.startswith(app)):
				foundApp = 1
				error = AdminApp.update(installedApp, 'app' , attrs)
				AdminConfig.save()		
				result = AdminApp.isAppReady(installedApp)
				while (result == "false"):
					### Wait 5 seconds before checking again
					time.sleep(5)
					result = AdminApp.isAppReady(installedApp)
				#endWhile
			#endIf
		#endFor
		if (foundApp == 0):
			#An informative message will be printed instead of an error
			print "  Info: "+app+" was not updated because the installed application could not be found - skipped"
		#endIf
		
	except:
		_type_, _value_, _tbck_ = sys.exc_info()
		error = `_value_`
		print "Error Installing Application "
		print "Error Message = "+error
		#indicate an error due to an unsuccessful installation
		returncode = 1
	#endTry
#endFor

# Notify the caller whether all applications were installed successfully
sys.exit(returncode)



