/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.query.SavedFilterUtils");

dojo.require("com.ibm.rdm.web.calm.CalmFilters");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.project.DashboardConstants");
dojo.require("com.ibm.rdm.web.project.SavedFilter");
dojo.require("com.ibm.rdm.web.suspicion.model.types");
dojo.require("com.ibm.rdm.web.suspicion.SuspectLinksUtil");
dojo.require("com.ibm.rdm.web.util.xml");
dojo.require("com.ibm.rdm.web.linkvalidity.LinkValidityUtils");

var CalmFilters		  = com.ibm.rdm.web.calm.CalmFilters;
var LinkValidityUtils = com.ibm.rdm.web.linkvalidity.LinkValidityUtils;
var RRMTypes		  = com.ibm.rdm.web.model.types.RRMTypes;
var LinkTypeUtils	  = com.ibm.rdm.web.query.linkTypeUtils;
var SuspectLinksUtil  = com.ibm.rdm.web.suspicion.SuspectLinksUtil;
var xml				  = com.ibm.rdm.web.util.xml;

var R1						   = "R1";
var R2						   = "R2";
var RDM_PARENT				   = "http://com.ibm.rdm/navigation#parent";
var RDM_TAG					   = "http://com.ibm.rdm/navigation#tag";
var VIEW_MODULE_CONTENT_COLUMN = "http://com.ibm.rdm/viewdata#moduleContentColumn";

var Namespaces = {
		clmFilter:			"http://com.ibm.rdm/viewdata#clmFilter",
		dc:					"http://purl.org/dc/elements/1.1/",
		dcterms:			"http://purl.org/dc/terms/",
		expansionLevel:		"http://com.ibm.rdm/viewdata#expansionLevel",
		gridColumn:			"http://com.ibm.rdm/viewdata#gridColumn",
		hierarchyLevel:		"http://com.ibm.rdm/viewdata#hierarchyLevel",
		owl:				"http://www.w3.org/2002/07/owl#",
		rdf:				"http://www.w3.org/1999/02/22-rdf-syntax-ns#",
		rm:					"http://www.ibm.com/xmlns/rdm/rdf/",
		rql:				"http://www.ibm.com/xmlns/rdm/rql/",
		rrmViewdata:		"http://com.ibm.rdm/viewdata#",
		traceAttributeData:	"http://com.ibm.rdm/viewdata#traceAttributeData"
	 },
	 _parsedFilter	   = null,
	 _componentURI	   = null,
	 _customAttributes = null,
	 _attributeTypes   = null,
	 _linkTypes		   = null;

com.ibm.rdm.web.query.SavedFilterUtils = {
	// summary:
	//		This singleton is used to parse saved filters

	///////////////////////////////////////////////////////////////////////////
	// Public API
	///////////////////////////////////////////////////////////////////////////
	
	parseFilterFromXML: function(filterObject, componentURI, customAttributes, attributeTypes, linkTypes) {
		// summary:
		//		Parses an XML object representing a saved filter (view), and returns a JSON object
		//		that contains most of the view's information. See how _parsedFilter gets
		//		initialized in _initParsedFilter() to understand the JSON's structure. 
		// description:
		//		This top-level function initializes the parser and the member variables. It
		//		then initiates parsing for all needed top-level nodes in the view.
		var view = xml.parser(filterObject, Namespaces).child("rdf:RDF").child("rm:View");
		if (!view) { return; }
		
		_componentURI	  = componentURI;
		_customAttributes = customAttributes;
		_attributeTypes	  = attributeTypes;
		_linkTypes		  = linkTypes;
		this._initParsedFilter();
		
		// parse the relevant top-level nodes
		_parsedFilter.displayBaseProperties = view.value("rm:displayBaseProperties");
		this._parseClientData(view.child("rrmViewdata:clientData"));
		this._parseRowQuery(view.child("rm:rowquery"));
		this._parseLinkQuery(view.child("rm:linkquery"));
		
		return _parsedFilter;
	},
	
	getTitleFromXML: function(filterObject) {
		// summary:
		//		Parses an XML object representing a saved filter (view), and returns the view's title.
		var view = xml.parser(filterObject, Namespaces).child("rdf:RDF").child("rm:View");
		if (!view) { return; }
		return view.value("dcterms:title");
	},
	
	getQueryURI: function(scope, baselineURI, processAreaContextURI, artifactInstanceURI, artifactTypeURI) {
		var queryURI = com.ibm.rdm.web.FRONTING_SERVICE_VIEWS + "?query=true&scope=" + scope;
		queryURI += (baselineURI) ? "&baseline=" + encodeURIComponent(baselineURI) : "";
		queryURI += (processAreaContextURI) ? "&processAreaContext=" + encodeURIComponent(processAreaContextURI) : "";
		queryURI += (artifactInstanceURI) ? "&module=" + encodeURIComponent(artifactInstanceURI) : "";
		queryURI += (artifactTypeURI) ? "&scopeArtifactType=" + encodeURIComponent(artifactTypeURI) : "";
		queryURI += (this._isPrivateScope(scope) || this._isCategoryScope(scope)) ? "&creator=USER" : "";
		return queryURI;
	},
	
	getMaterializedViewURI: function(viewURI) {
		var queryParamsIndex = viewURI.indexOf("?");
		if (queryParamsIndex !== -1) {
			viewURI = viewURI.replace(viewURI.substring(queryParamsIndex, viewURI.length), "");
		}
		return viewURI.replace(com.ibm.rdm.web.project.SavedFilter.ENDPOINT,
							   com.ibm.rdm.web.project.SavedFilter.MATERIALIZED_ENDPOINT);
	},
	
	isSavedView: function(viewURI) {
		return viewURI.indexOf(com.ibm.rdm.web.project.SavedFilter.MATERIALIZED_ENDPOINT) !== -1;
	},
	
	getScope: function(viewType) {
		var scope;
		if (viewType !== null) {
			switch (viewType) {
				case com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.COLLECTION:
					scope = com.ibm.rdm.web.project.SavedFilter.scope.COLLECTION;
					break;
				case com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE:
					scope = com.ibm.rdm.web.project.SavedFilter.scope.MODULE;
					break;
				default:
					scope = com.ibm.rdm.web.project.SavedFilter.scope.ARTIFACT;
					break;
			}
		} else {
			scope = com.ibm.rdm.web.project.SavedFilter.scope.ARTIFACT;
		}
		return scope;
	},
	
	
	///////////////////////////////////////////////////////////////////////////
	// Private API
	///////////////////////////////////////////////////////////////////////////
	
	_initParsedFilter: function() {
		// summary:
		//		Creates the base structure of the JSON object that will be returned.
		_parsedFilter = {
			folderURIs: [],
			tagURIs: [],
			suspicionProfileURIs: [],
			tagOperator: com.ibm.rdm.web.query.ViewServiceClient.TAG_OPERATORS.IsAnyOf,
			search: null,
			users: null,
			artifactFormat: null,
			artifactTypeConditionInfo: null,
			attributeConditionInfos: null,
			dateRange: null,
			attributes: null,
			calm: [],
			linkedArtifactConditionInfos: [],
			whereUsedConditionInfo: null,
			groupBy: null,
			orderBy: [],
			columns: [],
			displayedLinks: [],
			expansionRules: null,
			expansionLevel: -1, //'Off' by default
			displayBaseProperties: false // do not include base properties by default
		};
	},
	
	_parseClientData: function(clientDataNode) {
		// summary:
		//		Initiates parsing for all top-level nodes in the "rrmViewdata:clientData" node.
		if (!clientDataNode) { return; }
		
		var clientDataLength = clientDataNode.childElementCount,
			rdfNode			 = null,
			i				 = 1;
		
		for (; i <= clientDataLength; i++) {
			rdfNode = clientDataNode.child("rdf:_" + i);
			if (!rdfNode) { break; }
			
			rdfNode.forEachChild("rrmViewdata:gridColumn",	   this._parseGridColumn,	  this);
			rdfNode.forEachChild("rrmViewdata:clmFilter",	   this._parseClmFilter,	  this);
			rdfNode.forEachChild("rrmViewdata:expansionLevel", this._parseExpansionLevel, this);
			rdfNode.forEachChild("rrmViewdata:hierarchyLevel", this._parseHierarchyLevel, this);
		}
	},
	
	_parseRowQuery: function(rowQueryNode) {
		// summary:
		//		Initiates parsing for all nodes in the "rm:rowquery" node, a top-level node of the view.
		if (!rowQueryNode) { return; }
		
		rowQueryNode.forEachChild("rql:select",	this._parseRowQuerySelect, this);
		rowQueryNode.forEachChild("rql:where",	this._parseRowQueryWhere,  this);
		rowQueryNode.forEachChild("rql:sort",	this._parseRowQuerySort,   this);
	},
	
	_parseLinkQuery: function(linkQueryNode) {
		// summary:
		//		Parses the "rm:linkquery" node, a top-level node of the view.
		// description:
		//		All results will go into an array for _parsedFilter.expansionRules.
		if (!linkQueryNode) { return; }
		
		var linkRules	  = [],
			optsNode	  = null,
			linkQueryNode = linkQueryNode.child("rql:where").child("rdf:_1");
		if (!linkQueryNode || !linkQueryNode.child("rql:opts")) { return; }
		
		optsNode = linkQueryNode.child("rql:opts").child("rdf:Seq");
		if (!optsNode) { return; }
		
		optsNode.forEachChild("rdf:li", function(ruleNode){
			var valueNode =	ruleNode.child("rdf:value"),
				ifNode	  =	valueNode.child("rql:if"),
				thenNode  =	valueNode.child("rql:then"),
				R1URIs	  = [],
				R2URIs	  = [],
				linkURIs  = [];
			if(!ifNode) {
				valueNode.child("rql:e1").child("rdf:Seq").forEachChild("rdf:li", function(linkConstraint) {
					var linkDir = linkConstraint.value("rql:linkdir");
					var linkType = linkConstraint.child("rql:linktype").attribute("rdf:resource");
					if (linkType != "http://com.ibm.rdm/unmatchable_link") {
						linkURIs.push(((linkDir === "out") ? LinkTypeUtils.linkObjectSideToEnumAbout(linkType) :
							 LinkTypeUtils.linkSubjectSideToEnumAbout(linkType)));
					}
				}, this);
			} else {
				var ifNode1 = ifNode.child("rdf:_1"),
					ifNode2 = ifNode.child("rdf:_2");
				
				if (ifNode1) {
					ifNode = ifNode1.child("rql:e2").child("rdf:Seq");
					ifNode.forEachChild("rdf:li", function(artifact){
						var artifactURI = artifact.attribute("rdf:resource");
						if (ifNode1.value("rql:e1") === R1) {
							R1URIs.push(artifactURI);
						}
						else if (ifNode1.value("rql:e1") === R2) {
							R2URIs.push(artifactURI);
						}
					}, this);
				}
				
				if (ifNode2) {
					ifNode = ifNode2.child("rql:e2").child("rdf:Seq");
					ifNode.forEachChild("rdf:li", function(artifact){
						var artifactURI = artifact.attribute("rdf:resource");
						if (ifNode2.value("rql:e1") === R1) {
							R1URIs.push(artifactURI);
						}
						else if (ifNode2.value("rql:e1") === R2) {
							R2URIs.push(artifactURI);
						}
					}, this);
				}
				
				thenNode = thenNode.child("rql:e1").child("rdf:Seq");
				thenNode.forEachChild("rdf:li", function(linkNode){
					var linkDir	 = linkNode.value("rql:linkdir"),
						linkType = linkNode.child("rql:linktype").attribute("rdf:resource");
					
					linkURIs.push(((linkDir === "out") ? LinkTypeUtils.linkObjectSideToEnumAbout(linkType) :
														 LinkTypeUtils.linkSubjectSideToEnumAbout(linkType)));
				}, this);
			}
			
			if (R1URIs.length > 0 || R2URIs.length > 0 || linkURIs.length > 0) {
				// Note: We use 'unshift' rather than 'push' in the following as
				// we need to make sure to preserve the original ordering of the
				// "linkquery" related clauses.
				linkRules.unshift({
					"sourceArtifacts": R2URIs,
					"linkTypes":	   linkURIs,
					"targetArtifacts": R1URIs
				});
			}
		}, this);
		
		_parsedFilter.expansionRules = linkRules;
	},
	
	_parseGridColumn: function(gridColumnNode) {
		// summary:
		//		Parses the "rrmViewdata:gridColumn" node, inside the "rrmViewdata:clientData" node.
		// description:
		//		All results will go into the _parsedFilter.columns array.
		if (!gridColumnNode) { return; }
		
		var columnURI			= gridColumnNode.child("rrmViewdata:colURI").attribute("rdf:resource"),
			columnType			= gridColumnNode.value("rrmViewdata:colType"),
			traceAttrs			= null,
			traceAttrFormat		= null;
		
		//For the immediate we 'convert' the simple names used for system attributes into URIs to make things
		//more friendly/understandable on server for the future.
		//This Conversion behavior should go away in 2011 and resultSetGridView should handle exact URIs appropriately.
		switch (columnURI) {
			case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Identifier:
				columnURI = "id-col";
				break;
			case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Title:
				columnURI = "name";
				break;
			case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Description:
				columnURI = "description";
				break;
			case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.AlternateSpelling:
				columnURI = "alternateSpelling";
				break;
			case RRMTypes.WELL_KNOWN_OBJECT_TYPE:
				columnURI = "obj_type_col";
				break;
			case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ArtifactFormat:
				columnURI = "format";
				break;
			case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Contributor:
				columnURI = "mod-user";
				break;
			case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Creator:
				columnURI = "creator";
				break;
			case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ModifiedOn:
				columnURI = "mod-date";
				break;
			case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Created:
				columnURI = "created_date_col";
				break;
			case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Section:
				columnURI = "module-section-col";
				break;
			case com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["implementedBy"]:
				columnURI = "imp-by";
				break;
			case com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["trackedBy"]:
				columnURI = "track-by";
				columnType = "s";
				break;
			case com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["validatedBy"]:
				columnURI = "val-by";
				columnType = "s";
				break;
			case com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["affectedBy"]:
				columnURI = "aff-by";
				break;
			case RDM_TAG:
				columnURI = "tags";
				break;
			case RRMTypes.RDF_LOCKED_BY:
				columnURI = "lockedBy";
				break;
			case RRMTypes.RDF_LOCKED_ON:
				columnURI = "lockedOn";
				break;
			case VIEW_MODULE_CONTENT_COLUMN:
				columnURI = com.ibm.rdm.web.artifactgrid.ArtifactConstants.MODULE_CONTENT_COL;
				break;
			case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.CommentCount:
				columnURI = com.ibm.rdm.web.artifactgrid.ArtifactConstants.COMMENTS_COL;
				break;
			case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ReuseCount:
				columnURI = com.ibm.rdm.web.artifactgrid.ArtifactConstants.REUSEARTIFACTS_COL;
				break;
			case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ReuseCollections:
				columnURI = com.ibm.rdm.web.artifactgrid.ArtifactConstants.REUSEARTIFACTS_COLLECTIONS_COL;
				var traceAttrData = this._parseTraceAttributes(gridColumnNode);
				if (traceAttrData) { // For information on whether the titles should be trimmed or wrapped.
					traceAttrFormat = traceAttrData.format;
				}
				break;
			case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ReuseReviews:
				columnURI = com.ibm.rdm.web.artifactgrid.ArtifactConstants.REUSEARTIFACTS_REVIEWS_COL;
				var traceAttrData = this._parseTraceAttributes(gridColumnNode);
				if (traceAttrData) { // For information on whether the titles should be trimmed or wrapped.
					traceAttrFormat = traceAttrData.format;
				}
				break;
			case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ProcessArea:
				columnURI = com.ibm.rdm.web.artifactgrid.ArtifactConstants.TEAM_OWNERSHIP_COL;
				break;
			//Cases to handle tracked-by and affected-by col types, which are not quite system cols
			case "/track-by":
				columnURI = "track-by";
				break;
			case "/aff-by":
				columnURI = "aff-by";
				break;
			case "/imp-by":
				columnURI = "imp-by";
				break;
			case com.ibm.rdm.web.suspicion.model.types.SUSPECT_ENTRIES:
				if (SuspectLinksUtil.canShowSuspicionForProject(_componentURI)) {
					columnURI = com.ibm.rdm.web.artifactgrid.ArtifactConstants.SUSPICION_COL;
				} else {
					return;
				}
				break;
			case RRMTypes.LINK_VALIDITY:
				if (com.ibm.rdm.web.linkvalidity.LinkValidityUtils.isLinkValidityEnabledForProject(_componentURI)) {
					columnURI = com.ibm.rdm.web.artifactgrid.ArtifactConstants.LINK_VALIDITY_COL;
				} else {
					return;
				}
				break;
			default:
				// We only check for this when the column is a RM traceability link
				var traceAttrData = this._parseTraceAttributes(gridColumnNode);
				if(traceAttrData) {
					traceAttrs = traceAttrData.attributes;
					traceAttrFormat = traceAttrData.format;
				}
		}
		
		_parsedFilter.columns.push({
			colType:					  columnType,
			width:						  gridColumnNode.value("rrmViewdata:colWidth"),
			colURI:						  columnURI,
			columnTraceabilityAttrs:	  traceAttrs,
			columnTraceabilityAttrFormat: traceAttrFormat
		});
	},
	
	_parseClmFilter: function(clmFilterNode) {
		// summary:
		//		Parses the "rrmViewdata:clmFilter" node, inside the "rrmViewdata:clientData" node.
		// description:
		//		All results will go into the _parsedFilter.calm array.
		if (!clmFilterNode) { return; }
		
		var linkType = clmFilterNode.child("rrmViewdata:linkType").attribute("rdf:resource"),
			filterId = clmFilterNode.value("rrmViewdata:filterId");
		
		if (filterId.indexOf("#") >= 0) {
			filterId = filterId.split("#")[1];
		}
		_parsedFilter.calm.push(CalmFilters.getCalmFilterForLinkTypeAndFilter(linkType, filterId));
	},
	
	_parseExpansionLevel: function(expansionLevelNode) {
		// summary:
		//		Parses the "rrmViewdata:expansionLevel" node, inside the "rrmViewdata:clientData" node.
		if (!expansionLevelNode) { return; }
		_parsedFilter.expansionLevel = expansionLevelNode.value();
	},
	
	_parseHierarchyLevel: function(hierarchyLevelNode) {
		// summary:
		//		Parses the "rrmViewdata:hierarchyLevel" node, inside the "rrmViewdata:clientData" node.
		// note:
		//		When _parsedFilter.hierarchyLevel is set to a value, that value represents the level
		//		the hierarchy should be set to. When _parsedFilter.hierarchyLevel has NOT been set to
		//		a value, it means "Show Full Hierarchy".
		if (!hierarchyLevelNode) { return; }
		
		var hierarchyLevel = hierarchyLevelNode.value();
		if (hierarchyLevel) {
			hierarchyLevel = parseInt(hierarchyLevel);
		}
		_parsedFilter.hierarchyLevel = hierarchyLevel;
	},
	
	_parseRowQuerySelect: function(selectNode) {
		// summary:
		//		Parses the "rql:select" node, inside the "rm:rowquery" node.
		if (!selectNode) { return; }
		
		selectNode.forEachChild(function(selectNode){
			selectNode.forEachChild("rql:having", this._parseHaving, this);
		}, this);
	},
	
	_parseRowQueryWhere: function(whereNode) {
		// summary:
		//		Parses the "rql:where" node, inside the "rm:rowquery" node.
		// description:
		//		The following values in _parsedFilter will be set here: whereUsedConditionInfo,
		//		search, users, folderURIs, tagOperator, tagURIs, artifactTypeConditionInfo,
		//		dateRange, linkedArtifactConditionInfos, attributes, and attributeConditionInfos.
		if (!whereNode) { return; }
		
		var dateClauses				= {},
			durationClauses			= {},
			appliedCustomAttributes	= [],
			filterGroups			= {};
		
		whereNode.forEachChild(function(whereNode){
			var doesNotContain = false,
				isUnassigned   = false,
				isAll		   = false,
				filterGroup	   = null,
				operator	   = whereNode.value("rql:op"),
				e1Node		   = whereNode.child("rql:e1");
			
			// Check to see if it's a piece of link filter information.
			if (operator === "links") {
				filterGroup = whereNode.value("rql:to");
				
				if (!filterGroups[filterGroup]) {
					filterGroups[filterGroup] = {};
					filterGroups[filterGroup].linkValues = [];
				}
				else if (!filterGroups[filterGroup].linkValues) {
					filterGroups[filterGroup].linkValues = [];
				}
				
				if (whereNode.value("rql:not", "boolean")) {
					filterGroups[filterGroup].linkNot = true;
				}

				var linkOptsNode = whereNode.child("rql:opts");
				if (linkOptsNode) {
					linkOptsNode = linkOptsNode.child("rdf:Seq");
					
					linkOptsNode.forEachChild("rdf:li", function(linkInfoNode){
						linkInfoNode = linkInfoNode.child("rdf:value").child("rql:e1").child("rdf:Seq");
						
						linkInfoNode.forEachChild("rdf:li", function(linkNode){
							var linkDir	 = linkNode.value("rql:linkdir"),
								linkType = linkNode.child("rql:linktype").attribute("rdf:resource");
							
							filterGroups[filterGroup].linkValues.push(((linkDir === "in") ? LinkTypeUtils.linkObjectSideToEnumAbout(linkType) :
																							LinkTypeUtils.linkSubjectSideToEnumAbout(linkType)));
						}, this);
					}, this);
				}
			}
			else if (e1Node) {
				var typeObject				 = null,
					attributeDefinitionURI	 = null,
					wellKnownAttributeObject = null,
					isWhereUsedCondition	 = false,
					isLockOwnerQuery		 = false,
					isLockedBy				 = null;
				
				if (whereNode.value("rql:not", "boolean")) {
					doesNotContain = true;
				}
				if (whereNode.value("rql:all", "boolean")) {
					isAll = true;
				}
				if (operator === com.ibm.rdm.web.query.ViewServiceClient.ASSIGNMENT_OPERATORS.Unassigned) {
					isUnassigned = true;
				}
				
				//Only set typeObject if it's a node ref.
				if (e1Node.child("rql:object")) {
					typeObject	= e1Node;
					filterGroup	= e1Node.value("rql:object");
				}
				else{
					filterGroup = e1Node.value();
				}
				
				if (typeObject) {
					var fieldNode				 = typeObject.child("rql:field"),
						fieldValue				 = null,
						moduleBindingCounter	 = 0,
						lockedByUserCounter		 = 0,
						isBoundArtifactAttribute = false;
					
					if (fieldNode.child("rdf:Seq")) {
						fieldNode = fieldNode.child("rdf:Seq");
					}
					else {
						fieldValue = fieldNode.attribute("rdf:resource");
					}
					
					fieldNode.forEachChild("rdf:li", function(listNode){
						if ( (listNode.attribute("rdf:resource") === "http://www.ibm.com/xmlns/rdm/rdf/binding") ||
							 (listNode.attribute("rdf:resource") === RRMTypes.WELL_KNOWN_MODULE_TYPE) ) {
							moduleBindingCounter++;
						}
						else if ( (listNode.attribute("rdf:resource") === "http://www.ibm.com/xmlns/rdm/rdf/lock") ||
								  (listNode.attribute("rdf:resource") === "http://purl.org/dc/terms/creator") ) {
							lockedByUserCounter++;
						}
						else if ( (listNode.attribute("rdf:resource") === RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.BoundArtifact) ) {
							isBoundArtifactAttribute = true;
							return;
						}
					}, this);
					
					if (moduleBindingCounter === 2) {
						isWhereUsedCondition = true;
					}
					else if (lockedByUserCounter === 2) {
						isLockOwnerQuery = true;
					}
					else {
						// Note: The following 'if' statement is to account for the fact
						// that, within a module, a view may be filtering attributes that
						// are found within the bound artifact (see Defect 73678).
						if (isBoundArtifactAttribute) {
							fieldNode.forEachChild("rdf:li", function(listNode){
								if ( (listNode.attribute("rdf:resource") !== RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.BoundArtifact) ) {
									attributeDefinitionURI = listNode.attribute("rdf:resource");
									return;
								}
							}, this);
						}
						else {
							attributeDefinitionURI = fieldValue;
						}
						wellKnownAttributeObject = RRMTypes.getWellKnownAttributeObjectFromURI(
							attributeDefinitionURI,
							_componentURI
						);
					}
				}
				
				//valueObject may not exist, we may be simply checking for an attribute being set (in this case, notAttributeSetCheck == false).
				//In this case only the typeObject would exist
				var valueObject	= null,
					values		= [],
					value		= null,
					notAttributeSetCheck = false;
				if (whereNode.child("rql:e2") && whereNode.child("rql:e2").child("rdf:Seq")) {
					notAttributeSetCheck = true;
					valueObject = whereNode.child("rql:e2").child("rdf:Seq");
					valueObject.forEachChild("rdf:li", function(listNode){
						value = listNode.value() || listNode.attribute("rdf:resource");
						values.push(value);
					}, this);
				}
				else {
					value = whereNode.value("rql:e2");
					if (value) {
						notAttributeSetCheck = true;
						values.push(value.replace(/\"/g, ""));
					}
				}
				
				if (isLockOwnerQuery) {
					isLockedBy = values;
				}
				else {
					isLockedBy = null;
				}
				
				if (isWhereUsedCondition) {
					values.splice(0, 0, this._findEnumOperator(doesNotContain, isAll));
					_parsedFilter.whereUsedConditionInfo = {
						conditionType: com.ibm.rdm.web.query.conditionTypes.ModuleFormatInstancesCondition,
						attributeValues: values
					};
				}
				else if (wellKnownAttributeObject && values.length > 0 && wellKnownAttributeObject.sameAs) {
					switch(wellKnownAttributeObject.sameAs){
						case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Title:
							var searchFilter = {
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: wellKnownAttributeObject.attributeType,
								attributeValues: [
										this._findStringOperator(values[0], doesNotContain),
										this._cleanStringValue(values[0])
									],
								isEnum: false
							};
							
							if (filterGroup != R1){
								this._addToFilterGroup(filterGroups, filterGroup, "attributeConditionInfos", searchFilter, true);
							}
							else{
								_parsedFilter.search = searchFilter;
							}
							break;
						case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.AlternateSpelling:
							var altSpellingFilter = {
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: wellKnownAttributeObject.attributeType,
								attributeValues: [
										this._findStringOperator(values[0], doesNotContain),
										this._cleanStringValue(values[0])
									],
								isEnum: false
							};
							
							if (filterGroup != R1){
								this._addToFilterGroup(filterGroups, filterGroup, "attributeConditionInfos", altSpellingFilter, true);
							}
							else{
								appliedCustomAttributes.push(altSpellingFilter);
							}
							break;
						case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Contributor:
							var newValues =  [];
							newValues.push(this._findEnumOperator(doesNotContain, isAll));
							//Need a special case here to handle JSON not preserving proper relative URIs for users
							for (var curValue in values){
								newValues.push(values[curValue]);
							}
							
							var usersFilter = {
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: wellKnownAttributeObject.attributeType,
								attributeValues: newValues,
								isEnum: true
							};
							
							if (filterGroup != R1){
								this._addToFilterGroup(filterGroups, filterGroup, "attributeConditionInfos", usersFilter, true);
							}
							else{
								_parsedFilter.users = usersFilter;
							}
							break;
						case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ModifiedOn:
							if (!dateClauses[attributeDefinitionURI]) {
								dateClauses[attributeDefinitionURI] = {};
								dateClauses[attributeDefinitionURI][filterGroup] = [];
							}
							else{
								if (!dateClauses[attributeDefinitionURI][filterGroup]) {
									dateClauses[attributeDefinitionURI][filterGroup] = [];
								}
							}
							dateClauses[attributeDefinitionURI][filterGroup].push({
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: wellKnownAttributeObject.attributeType,
								value: values[0],
								operator: operator,
								sameAs: RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ModifiedOn
							});
							break;
						case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Creator:
							//we must rebuild the values array such that the operator is the first element
							var newValues =  [];
							newValues.push(this._findEnumOperator(doesNotContain, isAll));
							//Need a special case here to handle JSON not preserving proper relative URIs for users
							for (var curValue in values){
								newValues.push(values[curValue]);
							}
							var creator = {
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: wellKnownAttributeObject.attributeType,
								attributeValues: newValues,
								isEnum: true
							};
							
							if (filterGroup != R1){
								this._addToFilterGroup(filterGroups, filterGroup, "attributeConditionInfos", creator, true);
							}
							else{
								appliedCustomAttributes.push(creator);
							}
							break;
						case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Created:
							if (!dateClauses[attributeDefinitionURI]) {
								dateClauses[attributeDefinitionURI] = {};
								dateClauses[attributeDefinitionURI][filterGroup] = [];
							}
							else{
								if (!dateClauses[attributeDefinitionURI][filterGroup]) {
									dateClauses[attributeDefinitionURI][filterGroup] = [];
								}
							}
							dateClauses[attributeDefinitionURI][filterGroup].push({
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: wellKnownAttributeObject.attributeType,
								value: values[0],
								operator: operator,
								sameAs: RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Created
							});
							break;
						case RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Identifier:
							var operator = com.ibm.rdm.web.query.ViewServiceClient.ARTIFACT_ID_OPERATORS.IsAny,
								attributeValues = [operator].concat(values),
								identifier = {
									attributeDefinitionURI: attributeDefinitionURI,
									attributeType: RRMTypes.ATTRIBUTE_TYPES.IntegerType,
									attributeValues: attributeValues,
									isEnum: false
								};
							
							if (filterGroup != R1){
								this._addToFilterGroup(filterGroups, filterGroup, "attributeConditionInfos", identifier, true);
							} else {
								appliedCustomAttributes.push(identifier);
							}
							
							break;
					}
				}
				else if (attributeDefinitionURI === RRMTypes.RDF_LOCK || isLockedBy) {
					
					// this doesn't seem great, we cannot determine that there is specific "locked by"
					// criteria until after we have established that this is a lock existence query (well, we
					// probably could but this seems a safer change).  Locked by anyone merely queries
					// the existence of a lock so has no list of users.
					if (isLockedBy) {
						// so, if we have locked by list of users or current user, we need to remove
						// the more general "lock exists" we already added...
						var lockCondIdx;
						if (filterGroup != R1 && filterGroups[filterGroup]) {
							// find the index of the lock filter we made earlier and remove it.
							lockCondIdx = this._findLockFilter(filterGroups[filterGroup]["attributeConditionInfos"]);
							if (lockCondIdx > -1) {
								filterGroups[filterGroup]["attributeConditionInfos"].splice(lockCondIdx, 1);
							}
						} else {
							lockCondIdx = this._findLockFilter(appliedCustomAttributes);
							if (lockCondIdx > -1) {
								appliedCustomAttributes.splice(lockCondIdx, 1);
							}
						}
					} 
					// build the lock condition and add it in the *correct* place (WI:73595)
					var condition = this._getLockCondition(isLockedBy);
						
					if (filterGroup != R1){
						this._addToFilterGroup(filterGroups, filterGroup, "attributeConditionInfos", condition, true);
					} else {
						appliedCustomAttributes.push(condition);
					}
					 
				}
				else if (attributeDefinitionURI == RDM_PARENT){
					_parsedFilter.folderURIs = values;
				}
				else if (attributeDefinitionURI == RDM_TAG){
					_parsedFilter.tagOperator = this._findTagOperator(doesNotContain, isAll);
					_parsedFilter.tagURIs = values;
				}
				//When it's a piece of link filtering, typeobject would also be null
				else if (!typeObject){
					if (operator === "format") {
						//we must rerebuild the values array such that the operator is the first element
						var newValues =  [];
						newValues.push(this._findEnumOperator(doesNotContain, isAll));
						for (var curValue in values){
							if (values[curValue] == "\"Collection\"" || values[curValue] == "\"Text\"" || values[curValue] == "\"WrapperResource\"") {
								values[curValue] = values[curValue].substring(1,(values[curValue].length - 1));
							} else {
								values[curValue] = com.ibm.rdm.web.query.artifactFormatUtils.viewFormatValueToEnumAbout(
									values[curValue].substring(1,(values[curValue].length - 1)));
							}
							
							newValues.push(values[curValue]);
						}
						
						var formatFilter = {
							attributeDefinitionURI: RRMTypes.getWellKnownAttributeURI
											(_componentURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.ArtifactFormat),
							attributeType: RRMTypes.ATTRIBUTE_TYPES.ArtifactFormatsType,
							attributeValues: newValues,
							isEnum: true
						};
						
						if (filterGroup != R1){
							this._addToFilterGroup(filterGroups, filterGroup, "attributeConditionInfos", formatFilter, true);
						}
						else{
							appliedCustomAttributes.push(formatFilter);
						}
					}
					else {
						var newValues = [];
						newValues.push(this._findEnumOperator(doesNotContain, isAll));

						for (var curValue in values) {
							newValues.push(values[curValue]);
						}
						
						var typeFilter = {
							attributeValues: newValues,
							isEnum: true
						};
						
						if (filterGroup != R1){
							this._addToFilterGroup(filterGroups, filterGroup, "artifactTypeConditionInfo", typeFilter);
						}
						else{
							_parsedFilter.artifactTypeConditionInfo = typeFilter;
						}
					}
				}
				else if (_customAttributes[attributeDefinitionURI]) {
					var range = null;
					var attributeDefinition = _customAttributes[attributeDefinitionURI];
					if (attributeDefinition) {
						range = _attributeTypes[attributeDefinition.getRange()];
					}
					var sameAs = null, valueType = null;
					
					//in the first case it's a custom attribute, second case is special for artifactFormat (which
					//is in fact a well known)
					if (attributeDefinition) {
						sameAs = range.getSameAs();
						valueType = range.getValueType();
					}
					var isEnum = false;
					var newValues = [];
					
					var enumEntries = range.getEnumEntries();
					if (range && enumEntries && enumEntries.length > 0 ) {
						isEnum = true;
						newValues.push(this._findEnumOperator(doesNotContain, isAll));
					}
					
					for (var curValue in values) {
						newValues.push(values[curValue]);
					}
					
					var baseType = valueType;
					// Note: The attribute MAY be based on a custom attribute type,
					// in which case we can't just assume baseType should be set to
					// sameAs (as sameAs will be not be set).  However, if sameAs
					// isn't set we can't safely default to setting baseType to
					// valueType, due to the fact that if the underlying type is
					// an enum, the following will be true:
					//     isEnum will be set to true
					//     valueType will be set to "http://www.w3.org/2001/XMLSchema#int"
					// EXCEPT for workflow where the valueType is String.
					// and we need to end up in the 'default' switch case below
					// rather than the RRMTypes.ATTRIBUTE_TYPES.IntegerType switch
					// case (so that it's properly handled as an enum).
					if (sameAs && isEnum) {
						baseType = sameAs;
					}

					var getAttributeValuesArrayContainingOnlyOperator = function(){
						return (
							isUnassigned ? 
							[com.ibm.rdm.web.query.ViewServiceClient.ASSIGNMENT_OPERATORS.Unassigned] :
							(
								doesNotContain ?
								[com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.DoesNotExist] : 
								[com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.Exists]
							)
						);
					};

					switch (baseType) {
						case RRMTypes.ATTRIBUTE_TYPES.IntegerType:
							var localOperator = this._findNumberOperator(operator);
							
							var integerFilter = {
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: RRMTypes.ATTRIBUTE_TYPES.IntegerType,
								// If notAttributeSetCheck, proceed as normal. If not, we must evaluate if 
								// it's 'exists', 'does not exist' or 'is unassigned'.
								attributeValues: (
									(notAttributeSetCheck) ?
									// If the attribute is an enumeration (in which case the base type is integer),
									// then we won't have local operator.
									// Instead, the newValues object will contain the operator and the value.
									(localOperator ? ([localOperator, newValues[0]]) : newValues) :
									getAttributeValuesArrayContainingOnlyOperator()
								),
								isEnum: isEnum
							};
							
							if (filterGroup != R1){
								this._addToFilterGroup(filterGroups, filterGroup, "attributeConditionInfos", integerFilter, true);
							}
							else{
								appliedCustomAttributes.push(integerFilter);
							}
							break;
						case RRMTypes.ATTRIBUTE_TYPES.FloatType:
							var localOperator = this._findNumberOperator(operator);
							
							var floatFilter = {
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: RRMTypes.ATTRIBUTE_TYPES.FloatType,
								attributeValues: (
									(notAttributeSetCheck)  ? 
									[localOperator, newValues[0]] :
									getAttributeValuesArrayContainingOnlyOperator()
								),
								isEnum: isEnum
							};
							
							if (filterGroup != R1){
								this._addToFilterGroup(filterGroups, filterGroup, "attributeConditionInfos", floatFilter, true);
							}
							else{
								appliedCustomAttributes.push(floatFilter);
							}
							break;
						case RRMTypes.ATTRIBUTE_TYPES.BooleanType:
							newValues.splice(0, 0, com.ibm.rdm.web.query.ViewServiceClient.BOOLEAN_OPERATORS.Is);
							
							var boolFilter = {
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: RRMTypes.ATTRIBUTE_TYPES.BooleanType,
								attributeValues: (
									(notAttributeSetCheck)  ?
									// If value Object exists, proceed as normal. 
									newValues :
									// If not, we must evaluate if it's 'exists', 'does not exist' or 'is unassigned'.
									getAttributeValuesArrayContainingOnlyOperator()
								),
								isEnum: isEnum
							};
							if (filterGroup != R1){
								this._addToFilterGroup(filterGroups, filterGroup, "attributeConditionInfos", boolFilter, true);
							}
							else{
								appliedCustomAttributes.push(boolFilter);
							}
							break;
						case RRMTypes.ATTRIBUTE_TYPES.UserType:
							isEnum = true;
							newValues.splice(0, 0, this._findEnumOperator(doesNotContain, isAll, operator));
							
							var userFilter = {
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: RRMTypes.ATTRIBUTE_TYPES.UserType,
								attributeValues: (
									(notAttributeSetCheck)  ?
									// If value Object exists, proceed as normal.
									newValues :
									// If not, we must evaluate if it's 'exists', 'does not exist' or 'is unassigned'.
									getAttributeValuesArrayContainingOnlyOperator()
								),
								isEnum: isEnum
							};
							
							if (filterGroup != R1){
								this._addToFilterGroup(filterGroups, filterGroup, "attributeConditionInfos", userFilter, true);
							}
							else{
								appliedCustomAttributes.push(userFilter);
							}
							break;
						case RRMTypes.ATTRIBUTE_TYPES.ArtifactFormatsType:
							for (var i = 1; i < newValues.length; i++){
								newValues[i] = com.ibm.rdm.web.query.artifactFormatUtils.viewFormatValueToEnumAbout(newValues[i].split("#")[1]);
							}
						
							var formatFilter = {
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: RRMTypes.ATTRIBUTE_TYPES.ArtifactFormatsType,
								attributeValues: (notAttributeSetCheck)  ? newValues : //if value Object exists, proceed as normal. 
																	//If not, we must evaluate if it's an is vs. is not situation
									(doesNotContain ? [com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.DoesNotExist] : 
											[com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.Exists]),
								isEnum: isEnum
							};
							
							if (filterGroup != R1){
								this._addToFilterGroup(filterGroups, filterGroup, "attributeConditionInfos", formatFilter, true);
							}
							else{
								appliedCustomAttributes.push(formatFilter);
							}
							break;
						case RRMTypes.ATTRIBUTE_TYPES.StringType:
							var stringFilter,
								isWorkflow = false;
							
							if (attributeDefinition) {
								isWorkflow = attributeDefinition.isWorkflow();
							}
							
							// If it's a workflow type, then handle it as an enumeration,
							// where the newValues object will contain the operator and the value.
							if (isWorkflow) {
								stringFilter = {
									attributeDefinitionURI: attributeDefinitionURI,
									attributeType: RRMTypes.ATTRIBUTE_TYPES.StringType,
									attributeValues: (
										newValues
									),
									isEnum: isEnum
								};
							} else {
								stringFilter = {
									attributeDefinitionURI: attributeDefinitionURI,
									attributeType: RRMTypes.ATTRIBUTE_TYPES.StringType,
									attributeValues: (
										(notAttributeSetCheck)  ?
										// If value Object exists, proceed as normal.
										[
											this._findStringOperator(newValues[0], doesNotContain),
											this._cleanStringValue(newValues[0])
										] :
										// If not, we must evaluate if it's 'exists', 'does not exist' or 'is unassigned'.
										getAttributeValuesArrayContainingOnlyOperator()
									),
									isEnum: isEnum
								};
							}
							
							if (filterGroup != R1){
								this._addToFilterGroup(filterGroups, filterGroup, "attributeConditionInfos", stringFilter, true);
							}
							else{
								appliedCustomAttributes.push(stringFilter);
							}
							break;
						case RRMTypes.ATTRIBUTE_TYPES.DurationType:
							if (!durationClauses[attributeDefinitionURI]) {
								durationClauses[attributeDefinitionURI] = {};
								durationClauses[attributeDefinitionURI][filterGroup] = [];
							}
							else{
								if (!durationClauses[attributeDefinitionURI][filterGroup]){
									durationClauses[attributeDefinitionURI][filterGroup] = [];
								}
							}
							durationClauses[attributeDefinitionURI][filterGroup].push({
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: RRMTypes.ATTRIBUTE_TYPES.DurationType,
								value: newValues[0],
								operator: operator,
								doesNotContain: doesNotContain
							});
							break;
						case RRMTypes.ATTRIBUTE_TYPES.DateType:
							if (!dateClauses[attributeDefinitionURI]) {
								dateClauses[attributeDefinitionURI] = {};
								dateClauses[attributeDefinitionURI][filterGroup] = [];
							}
							else{
								if (!dateClauses[attributeDefinitionURI][filterGroup]) {
									dateClauses[attributeDefinitionURI][filterGroup] = [];
								}
							}
							dateClauses[attributeDefinitionURI][filterGroup].push({
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: RRMTypes.ATTRIBUTE_TYPES.DateType,
								value: newValues[0],
								operator: operator,
								doesNotContain: doesNotContain
							});
							break;
						case RRMTypes.ATTRIBUTE_TYPES.TimeType:
							if (!dateClauses[attributeDefinitionURI]) {
								dateClauses[attributeDefinitionURI] = {};
								dateClauses[attributeDefinitionURI][filterGroup] = [];
							}
							else{
								if (!dateClauses[attributeDefinitionURI][filterGroup]) {
									dateClauses[attributeDefinitionURI][filterGroup] = [];
								}
							}
							dateClauses[attributeDefinitionURI][filterGroup].push({
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: RRMTypes.ATTRIBUTE_TYPES.TimeType,
								value: newValues[0],
								operator: operator,
								filterGroup: filterGroup,
								doesNotContain: doesNotContain
							});
							break;
						case RRMTypes.ATTRIBUTE_TYPES.DateTimeType:
							if (!dateClauses[attributeDefinitionURI]) {
								dateClauses[attributeDefinitionURI] = {};
								dateClauses[attributeDefinitionURI][filterGroup] = [];
							}
							else{
								if (!dateClauses[attributeDefinitionURI][filterGroup]) {
									dateClauses[attributeDefinitionURI][filterGroup] = [];
								}
							}
							dateClauses[attributeDefinitionURI][filterGroup].push({
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: RRMTypes.ATTRIBUTE_TYPES.DateTimeType,
								value: newValues[0],
								operator: operator,
								filterGroup: filterGroup,
								doesNotContain: doesNotContain
							});
							break;
						default:
							var customFilter ={
								attributeDefinitionURI: attributeDefinitionURI,
								attributeType: RRMTypes.ATTRIBUTE_TYPES.StringType,
								
								attributeValues: (
									(notAttributeSetCheck)  ?
									// If value Object exists, proceed as normal.
									newValues :
									// If not, we must evaluate if it's 'exists', 'does not exist' or 'is unassigned'.
									getAttributeValuesArrayContainingOnlyOperator()
								),
								isEnum: isEnum
							};
							
							if (filterGroup != R1){
								this._addToFilterGroup(filterGroups, filterGroup, "attributeConditionInfos", customFilter, true);
							}
							else{
								appliedCustomAttributes.push(customFilter);
							}
							break;
					}
				}
			}
		}, this);
		
		//Iterate over what we know of date bits, since they may have multiple clauses.
		for (var dateClause in dateClauses) {
			for (var currentGroup in dateClauses[dateClause]) {
				var currentDateAttribute = this._composeDateAttribute(dateClauses[dateClause][currentGroup]);
				if (currentGroup != R1) {
					this._addToFilterGroup(filterGroups, currentGroup, 
							"attributeConditionInfos", currentDateAttribute, true);
				}
				else {
					//If the dateclause contains sameAs that's modifiedOn, then set as modified on
					if (dateClauses[dateClause][currentGroup][0].sameAs &&
							dateClauses[dateClause][currentGroup][0].sameAs ==
								RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ModifiedOn)
					{
						_parsedFilter.dateRange = currentDateAttribute;
					} else {
						appliedCustomAttributes.push(currentDateAttribute);
					}
				}
			}
		}
		
		//Iterate over what we know of duration bits, since they may have multiple clauses.
		for (var durationClause in durationClauses){
			for (var currentGroup in durationClauses[durationClause]) {
				var currentDurationClause = this._composeDurationAttribute(durationClauses[durationClause][currentGroup]);
				if (currentGroup != R1) {
					this._addToFilterGroup(filterGroups, currentGroup, "attributeConditionInfos", currentDurationClause, true);
				}
				else {
					appliedCustomAttributes.push(currentDurationClause);
				}
			}
		}
		
		//Utilize any links information that was derived from the saved filter
		for (var linkInfo in filterGroups){
			var currentLinkParams = {};
			var currentLinkValues = filterGroups[linkInfo].linkValues;
			
			//If we have both subject and object, remove and change to bidirectional
			if (currentLinkValues){
				if (currentLinkValues.length > 1) {
					currentLinkValues = this._reduceAllDirectionalLinksThatAreReallyBidirectional(currentLinkValues);
				}
				
				currentLinkValues.splice(0,0,
					filterGroups[linkInfo].linkNot ? 
					com.ibm.rdm.web.query.ViewServiceClient.LINK_OPERATORS.DoesNotExist : 
					com.ibm.rdm.web.query.ViewServiceClient.LINK_OPERATORS.Exists
				);
				
				currentLinkParams["linkConditionInfo"] = {
					conditionType: com.ibm.rdm.web.query.conditionTypes.LinkTypeCondition,
					attributeValues: currentLinkValues
				};
			}
			
			var currentArtifactTypes = filterGroups[linkInfo]["artifactTypeConditionInfo"];
			if (currentArtifactTypes){
				currentLinkParams["artifactTypeConditionInfo"] = currentArtifactTypes;
				currentLinkParams["artifactTypeConditionInfo"].conditionType =
					com.ibm.rdm.web.query.conditionTypes.ArtifactTypeCondition;
			}
			
			var currentCustomAttributes = filterGroups[linkInfo]["attributeConditionInfos"];
			if (currentCustomAttributes && currentCustomAttributes.length > 0){
				currentLinkParams["attributeConditionInfos"] = [];
				for (var customAttrib in currentCustomAttributes){
					currentLinkParams["attributeConditionInfos"].push(
						currentCustomAttributes[customAttrib]
					);
				}
			}
			
			_parsedFilter.linkedArtifactConditionInfos.push(currentLinkParams);
		}
		
		_parsedFilter.attributes			  = appliedCustomAttributes;
		_parsedFilter.attributeConditionInfos = this._getAttributeConditionInfos(_parsedFilter);
	},
	
	_parseRowQuerySort: function(sortNode) {
		// summary:
		//		Parses the "rql:sort" node, inside the "rm:rowquery" node.
		// description:
		//		The following values in _parsedFilter will be set here: groupBy and orderBy.
		if (!sortNode) { return; }
		
		sortNode.forEachChild(function(sortNode){
			var notSort		 = sortNode.value("rql:not", "boolean"),
				order		 = sortNode.value("rql:order"),
				objFieldNode = sortNode.child("rql:objField"),
				sortOn		 = objFieldNode ? objFieldNode.child("rql:field").attribute("rdf:resource") : null;				
			
			if (notSort) {
				_parsedFilter.groupBy = {
					groupType: "custom",
					groupURI: sortOn
				};
				_parsedFilter.orderBy.push({
					groupType: "custom",
					def: sortOn,
					order: "d"
				});
			}
			else {
				_parsedFilter.orderBy.push({
					sortType: "custom",
					def: sortOn,
					order: order === "desc" ? "d" : "a"
				});
			}
		}, this);
	},
	
	_parseHaving: function(havingNode) {
		// summary:
		//		Parses the "rql:having" node, inside the "rql:select" node, inside the "rm:rowquery" node.
		// description:
		//		The following values in _parsedFilter will be set here: suspicionProfileURIs.
		if (!havingNode) { return; }
		
		var profileURIListNode = havingNode.child("rdf:_1").child("rql:e2").child("rdf:Seq"),
			profileURI		   = null,
			profileURIs		   = [];
		
		profileURIListNode.forEachChild("rdf:li", function(profileURINode){
			profileURI = profileURINode.attribute("rdf:resource");
			if (profileURI) {
				profileURIs.push(profileURI);
			}
		}, this);
		
		dojo.publish(SuspectLinksUtil.EVENT_TOPICS.profilesChangedFromFilter, [{
			source: SuspectLinksUtil.EVENT_SOURCE.savedFilter,
			projectContext: _componentURI,
			suspicionProfileURIs: profileURIs
		}]);
		_parsedFilter.suspicionProfileURIs = profileURIs;
	},
	
	_parseTraceAttributes: function(gridColumnNode) {
		// summary:
		//		Parses any trace attributes that are in the provided grid column node.
		// description:
		//		The trace attribute data will be returned to function _parseGridColumn(...).
		if (!gridColumnNode) { return; }
		
		var traceAttributeData	   = { attributes: [], format: {} },
			traceAttributeDataNode = gridColumnNode.child("rrmViewdata:traceAttributeData");
		if (!traceAttributeDataNode) { return; }
		
		var parseTraceAttribute = function(traceAttributeNode) {
			traceAttributeData.attributes.push({
				uri: traceAttributeNode.child("rrmViewdata:attrURI").attribute("rdf:resource")
			});
		};
		
		var parseTraceAttributeFormat = function(traceAttributeFormatNode) {
			traceAttributeData.format.showTitle			= traceAttributeFormatNode.value("rrmViewdata:showTitle", "boolean");
			traceAttributeData.format.showIcon			= traceAttributeFormatNode.value("rrmViewdata:showIcon", "boolean");
			traceAttributeData.format.showAttributeName	= traceAttributeFormatNode.value("rrmViewdata:showAttributeName", "boolean");
			traceAttributeData.format.trimWordsToFit	= traceAttributeFormatNode.value("rrmViewdata:trimWordsToFit", "boolean");
		};
		
		traceAttributeDataNode.forEachChild(function(traceAttributeDataNode){
			traceAttributeDataNode.forEachChild("rrmViewdata:traceAttribute",		parseTraceAttribute,	   this);
			traceAttributeDataNode.forEachChild("rrmViewdata:traceAttributeFormat",	parseTraceAttributeFormat, this);
		}, this);

		return traceAttributeData;
	},
	
	_isPrivateScope: function(scope) {
		return scope === com.ibm.rdm.web.project.SavedFilter.scope.PRIVATE
				|| scope === com.ibm.rdm.web.project.SavedFilter.scope.COLLECTION_PRIVATE
				|| scope === com.ibm.rdm.web.project.SavedFilter.scope.MODULE_PRIVATE;
	},

	_isCategoryScope: function(scope) {
		return scope === com.ibm.rdm.web.project.SavedFilter.scope.ARTIFACT
				|| scope === com.ibm.rdm.web.project.SavedFilter.scope.COLLECTION
				|| scope === com.ibm.rdm.web.project.SavedFilter.scope.MODULE;
	},
	
	_getLockCondition: function(/*Array?*/lockedBy) {
		// Summary:
		//		Builds a condition object for the lock
		//		criteria 
		// lockedBy:
		//		An optional array of users who own the lock
		
		var attributeValues = [],
			isLockedByCurrentUser = false;
		
		if (!lockedBy) {
			// there is only an existence operator i.e. "is any user"
			attributeValues = [com.ibm.rdm.web.query.ViewServiceClient.LOCKEDBY_OPERATORS.IsAnyUser];
		} else {
			// otherwise there are one or more users..
			dojo.forEach(lockedBy, function(user) {
				// we need to work out whether or not the special "current user"
				// operator is used.
				if (user && user.indexOf(RRMTypes.RDF_CURRENT_USER) > -1) {
					isLockedByCurrentUser = true;
				}
			});
			
			
			if (!isLockedByCurrentUser) {
				attributeValues = [com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsAny].concat(lockedBy);
			} else {
				attributeValues = [com.ibm.rdm.web.query.ViewServiceClient.LOCKEDBY_OPERATORS.IsCurrentUser];
			}
		}
		
		return {
			attributeDefinitionURI: RRMTypes.RDF_LOCKED_BY,
			attributeType: RRMTypes.ATTRIBUTE_TYPES.UserType,
			attributeValues: attributeValues,
			isEnum: true
		};

	},

	_reduceAllDirectionalLinksThatAreReallyBidirectional: function(currentLinkValues) {
		// summary:
		//		Return an array of link enum URIs that represent all values
		//		currently found in currentLinkValues, but with those links
		//		that actually represent both ends of bidirectional links
		//		"collapsed" into a single link enum URI that represents a
		//		bidirectional link.
		// description:
		//		A "bidirectional" link is a link in which separate roles
		//		have not been assigned for the link directions (subject to
		//		object and object to subject).  For such links, the
		//		link chooser would present only a single link for choosing,
		//		(e.g. "mylink" [assuming "mylink" doesn't have roles assigned
		//		to either end]) rather than multiple links (e.g. "Parent Of",
		//		"Child Of" for decomposition links).  However, when the view
		//		query is issued for a bidirectional link, we actually
		//		explicitly specify both ends of these bidirectional links.
		//		When a query is turned into a saved view, this breakout of
		//		the bidirectional link into the specific role based link
		//		URIs baked into the saved view.  This function is called to
		//		"collapse" these bidirectional link URIs back into the single
		//		link URI that the the user originally specified via the link
		//		chooser.
		// example:
		//		Suppose the user has created a bidirectional "mylink" link
		//		type and has used this to create a:
		//		    Link type exists Child Of, mylink
		//		In this case currentLinkValues would be something like this
		//		[
		//		    "https://spud.com:9444/rdm/types/_-Dl7AO2HEeGcpcwdvyDWgQ#object",
		//		    "https://spud.com:9444/rdm/types/_-Dl7AO2HEeGcpcwdvyDWgQ#subject",
		//		    "http://www.ibm.com/xmlns/rdm/types/Decomposition#subject"
		//		]
		//		and, when done, this function would return:
		//		[
		//		    "https://spud.com:9444/rdm/types/_-Dl7AO2HEeGcpcwdvyDWgQ#bidirectional",
		//		    "http://www.ibm.com/xmlns/rdm/types/Decomposition#subject"
		//		]

		var reducedLinkValues = [];
		var mapLinkURIToBoolean = {};
		var mapSameAsUriToLinkType = {};
		var mapOfAlreadyAccountedForBidirectionalUris = {};
		var linkType;
		// Fill mapSameAsUriToLinkType with appropriate values.
		for (var linkTypeURI in _linkTypes) {
			linkType = _linkTypes[linkTypeURI];
			var sameAs = linkType.getSameAs();
			if (sameAs) {
				mapSameAsUriToLinkType[sameAs] = linkType;
			}
		}
		// Fill reducedLinkValues with appropriate values.
		dojo.forEach(
			currentLinkValues,
			function(linkValue){
				var linkResourceURI =
					LinkTypeUtils.enumAboutToLinkResourceURI(linkValue);
				linkType = (
					mapSameAsUriToLinkType[linkResourceURI] ||
					_linkTypes[linkResourceURI]
				);
				if (linkType && linkType.isBidirectional()) {
					// The link type is bidirectional ... so we only add a single
					// "bidirectional" URI to the reducedLinkValues and keep track
					// of things (via mapOfAlreadyAccountedForBidirectionalUris)
					// so that we skip over the subsequent URI we'll hit for the
					// "other end" of this bidirectional link type.
					if (!mapOfAlreadyAccountedForBidirectionalUris[linkResourceURI]) {
						var enumAbout =
							LinkTypeUtils.bidirectionalLinkToEnumAbout(
								linkResourceURI
							);
						reducedLinkValues.push(enumAbout);
						mapOfAlreadyAccountedForBidirectionalUris[linkResourceURI] = true;
					}
				} else {
					// The link type is unidirectional ... just add it to the
					// reducedLinkValues.
					reducedLinkValues.push(linkValue);
				}
			},
			this
		);
		return reducedLinkValues;
	},

	_findLockFilter: function(filterArray) {
		var lockConditionIdx = -1;
		dojo.forEach(filterArray, function(thing, idx) {
			if (thing.attributeDefinitionURI === RRMTypes.RDF_LOCKED_BY) {
				lockConditionIdx = idx;
			}
		});
		
		return lockConditionIdx;
	},

	_addToFilterGroup: function(filterGroups, /*String*/filterGroup, /*String*/key, /*Object*/filterParams, /*?Boolean*/keyIsArray){
		if (!filterGroups[filterGroup]){
			filterGroups[filterGroup] = {};
		}
		if (keyIsArray){
			if (!filterGroups[filterGroup][key]){
				filterGroups[filterGroup][key] = [];
			}
			filterGroups[filterGroup][key].push(filterParams);
		}
		else{
			filterGroups[filterGroup][key] = filterParams;
		}
	},
	
	_getAttributeConditionInfos: function(filters) {
		// summary:
		//		Return an array that will represent the "attributes" property
		//		that will be passed in a call to the ViewServiceClient's
		//		performViewRequest function.
		var attributeConditionInfos = [];
		// Add an attribute for the "Limit by name" filter, if needed.
		if (_parsedFilter.search) {
			attributeConditionInfos.push(_parsedFilter.search);
		}
		// Add an attribute for the "Limit by date" filter, if needed.
		if (_parsedFilter.dateRange) {
			attributeConditionInfos.push(_parsedFilter.dateRange);
		}
		// Add an attribute for the "Limit by user" filter, if needed.
		if (_parsedFilter.users) {
			attributeConditionInfos.push(_parsedFilter.users);
		}
		// Add attributes for the "Limit by attribute" filter, if needed.
		var otherAttributeConditionInfos = _parsedFilter.attributes;
		if (otherAttributeConditionInfos) {
			attributeConditionInfos = attributeConditionInfos.concat(otherAttributeConditionInfos);
		}
		return attributeConditionInfos;
	},
	
	_findEnumOperator: function(isNot, isAll, /*String?*/operator) {
		var operatorToUse = com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.Exists;
		if (operator === "exists" && isNot) {
			operatorToUse = com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.DoesNotExist;
		}
		else if (isNot) {
			if (isAll) {
				operatorToUse = com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAll;
			}
			else {
				operatorToUse = com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAny;
			}
		}
		else {
			if (isAll) {
				operatorToUse = com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsAll;
			}
			else {
				operatorToUse = com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsAny;
			}
		}
		return operatorToUse;
	},

	_findTagOperator: function(isNot, isAll) {
		var operator;
		if (isNot) {
			if (isAll) {
				operator = com.ibm.rdm.web.query.ViewServiceClient.TAG_OPERATORS.IsNotAllOf;
			}
			else {
				operator = com.ibm.rdm.web.query.ViewServiceClient.TAG_OPERATORS.IsNotAnyOf;
			}
		}
		else {
			if (isAll) {
				operator = com.ibm.rdm.web.query.ViewServiceClient.TAG_OPERATORS.IsAllOf;
			}
			else {
				operator = com.ibm.rdm.web.query.ViewServiceClient.TAG_OPERATORS.IsAnyOf;
			}
		}
		return operator;
	},
	
	_composeDurationAttribute: function(durations){
		if (durations.length === 1){
			var durationOperator = null,
				isExistsOrUnassigned = false;
			
			switch (durations[0].operator){
				case "<":
					durationOperator = com.ibm.rdm.web.query.ViewServiceClient.DURATION_OPERATORS.IsLessThan;
					break;
				case ">":
					durationOperator = com.ibm.rdm.web.query.ViewServiceClient.DURATION_OPERATORS.IsGreaterThan;
					break;
				case "=":
					durationOperator = com.ibm.rdm.web.query.ViewServiceClient.DURATION_OPERATORS.Is;
					break;
				case ">=":
					durationOperator = com.ibm.rdm.web.query.ViewServiceClient.DURATION_OPERATORS.IsBetween;
					break;
				case "exists":
					if (durations[0].doesNotContain) {
						durationOperator = com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.DoesNotExist;
					} else {
						durationOperator = com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.Exists;
					}
					isExistsOrUnassigned = true;
					break;
				case "unassigned":
					durationOperator = com.ibm.rdm.web.query.ViewServiceClient.ASSIGNMENT_OPERATORS.Unassigned;
					isExistsOrUnassigned = true;
					break;
			}
			
			return({
				attributeDefinitionURI: durations[0].attributeDefinitionURI,
				attributeType: durations[0].attributeType,
				attributeValues: isExistsOrUnassigned ? [durationOperator] : [durationOperator, durations[0].value],
				isEnum: false
			});
		}
		else if (durations.length === 2) {
			var durationOperator = com.ibm.rdm.web.query.ViewServiceClient.DATETIME_OPERATORS.IsBetween;

			// Note: See Defect 71411 Comment 7 for why we have to do the following
			// if/else when assigning values for duration1 and duration2.
			var duration1, duration2;
			if (durations[0].operator === ">=") {
				duration1 = durations[0].value;
				duration2 = durations[1].value;
			} else {
				duration1 = durations[1].value;
				duration2 = durations[0].value;
			}
			
			return ({
				attributeDefinitionURI: durations[1].attributeDefinitionURI,
				attributeType: durations[1].attributeType,
				attributeValues: [
					durationOperator,
					duration1,
					duration2
				],
				isEnum: false
			});
		}
	},
	
	_composeDateAttribute: function(dates){
		/*Boolean*/ var onlyTime = (dates[0].attributeType === 
				RRMTypes.ATTRIBUTE_TYPES.TimeType ? true : false);
		
		if (dates.length === 1){
			var dateOperator = null,
				isExistsOrUnassigned = false;
			
			switch (dates[0].operator){
				case "<=":
					dateOperator = com.ibm.rdm.web.query.ViewServiceClient.DATETIME_OPERATORS.IsBefore;
					break;
				case ">=":
					dateOperator = com.ibm.rdm.web.query.ViewServiceClient.DATETIME_OPERATORS.IsAfter;
					break;
				case "=":
					if (dates[0].doesNotContain) {
						dateOperator = com.ibm.rdm.web.query.ViewServiceClient.DATE_OPERATORS.IsNot;
					} else {
						dateOperator = com.ibm.rdm.web.query.ViewServiceClient.DATE_OPERATORS.Is;
					}
					break;
				case "exists":
					if (dates[0].doesNotContain) {
						dateOperator = com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.DoesNotExist;
					} else {
						dateOperator = com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.Exists;
					}
					isExistsOrUnassigned = true;
					break;
				case "unassigned":
					dateOperator = com.ibm.rdm.web.query.ViewServiceClient.ASSIGNMENT_OPERATORS.Unassigned;
					isExistsOrUnassigned = true;
					break;
			}
			
			if (!isExistsOrUnassigned) {
				if (onlyTime){
					var splitTime = dates[0].value.split(":");
					dates[0].value = new Date(2000, 1, 1, splitTime[0], splitTime[1], splitTime[2]);
				}
				else{
					var date = dojo.date.stamp.fromISOString(dates[0].value);
					// Note: It MAY be the case that dates[0].value doesn't represent a
					// date, but instead represents a duration (e.g. "PT2H") or a string
					// value of "today" or "yesterday".
					dates[0].value = date || dates[0].value;
				}
			}
			
			return({
				attributeDefinitionURI: dates[0].attributeDefinitionURI,
				attributeType: dates[0].attributeType,
				attributeValues: isExistsOrUnassigned ? [dateOperator] : [dateOperator, dates[0].value],
				isEnum: false
			});
		}
		if (dates.length === 2){
			var dateOperator = com.ibm.rdm.web.query.ViewServiceClient.DATETIME_OPERATORS.IsBetween;
			
			if (onlyTime && dates[0].operator !== "exists"){
				var splitTime1 = dates[0].value.split(":");
				var date1 = new Date(2000, 1, 1, splitTime1[0], splitTime1[1], splitTime1[2]);
				var splitTime2 = dates[1].value.split(":");
				var date2 = new Date(2000, 1, 1, splitTime2[0], splitTime2[1], splitTime2[2]);
			}
			else{
				var date1 = dojo.date.stamp.fromISOString(dates[0].value);
				var date2 = dojo.date.stamp.fromISOString(dates[1].value);
			}
			
			//Determine if timeframe between the two dates is exactly 24 hours. If it is, this should be 'Is' rather than
			//'IsBetween'
			var dateDiff = date2.getTime() - date1.getTime();
			if (dateDiff === 86400000 || dateDiff === -86400000) {
				dateOperator = com.ibm.rdm.web.query.ViewServiceClient.DATETIME_OPERATORS.Is;
			}
				
			//Swap the dates around if they're in the wrong order
			if (dateDiff < 0){
				var tempDate = dojo.clone(date1);
				date1 = dojo.clone(date2);
				date2 = dojo.clone(tempDate);
			}
			
			return ({
				attributeDefinitionURI: dates[1].attributeDefinitionURI,
				attributeType: dates[1].attributeType,
				attributeValues: [
					dateOperator,
					date1,
					(dateOperator === com.ibm.rdm.web.query.ViewServiceClient.DATETIME_OPERATORS.Is) ? null : date2
				],
				isEnum: false
			});
		}
	},
	
	_findNumberOperator: function(operator){
		switch (operator) {
			case '=':
				return com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.Is;
			case '!=':
				return com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.IsNot;
			case '&gt;=':
			case '>=':
				return com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.IsGreaterThanOrEqualTo;
			case '&lt;=':
			case '<=':
				return com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.IsLessThanOrEqualTo;
		}
	},
	
	_findStringOperator: function(myString, doesNotContain) {
		if (doesNotContain){
			return com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.DoesNotContain;
		}
		else{
			if ((myString.indexOf('^.*') === 0) && (myString.indexOf('.*$') !== -1)) {
				return com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.Contains;
			}
			else if ((myString.indexOf('^') === 0) && (myString.indexOf('.*$') !== -1)) {
				return com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.StartsWith;
			}
			else if ((myString.indexOf('^.*') === 0) && (myString.indexOf('$') !== -1)) {
				return com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.EndsWith;
			}
			else if ((myString.indexOf('^') === 0) && (myString.indexOf('$') !== -1)) {
				return com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.Is;
			}
		}
	},
	
	_cleanStringValue: function(myString){
		var returnString = null;

		switch(this._findStringOperator(myString, false)){
			case com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.Contains:
				returnString = myString.replace("^.*", "").replace(".*$", "");
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.StartsWith:
				returnString = myString.replace("^", "").replace(".*$", "");
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.EndsWith:
				returnString = myString.replace("^.*", "").replace("$", "");
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.Is:
				returnString = myString.replace("^", "").replace("$", "");
				break;
		}
		
		return returnString;	
	}
};


})();
